#include "depthCalculator.h"
#include "Openvis3d.h"
#include "OpenCVImageAdapter.h"


IplImage*imgLeftDisparity ;//		= cvCreateImage(sz,IPL_DEPTH_64F,1);
IplImage*imgRightDisparity 	;//		= cvCreateImage(sz,IPL_DEPTH_64F,1);
IplImage*imgLeftOcclusions 	;//		= cvCreateImage(sz,IPL_DEPTH_64F,1);
IplImage*imgRightOcclusions ;//		= cvCreateImage(sz,IPL_DEPTH_64F,1);

//wrap all the input and output images in OpenCVImageAdapter, so that they can be
//accessed by OpenVis3D
OpenCVImageAdapter*ovaImg1 ;//	= new OpenCVImageAdapter(left.getCvImage());
OpenCVImageAdapter*ovaImg2 ;//	= new OpenCVImageAdapter(right.getCvImage());
OpenCVImageAdapter*ovaLeftDisp ;//	= new OpenCVImageAdapter(imgLeftDisparity);
OpenCVImageAdapter*ovaRightDisp ;//	= new OpenCVImageAdapter(imgRightDisparity);
OpenCVImageAdapter*ovaLeftOcc ;//	= new OpenCVImageAdapter(imgLeftOcclusions);
OpenCVImageAdapter*ovaRightOcc ;//	= new OpenCVImageAdapter(imgRightOcclusions);



depthCalculator::depthCalculator(int width, int height){
	
	CvSize sz;
	sz.height = height;
	sz.width  = width;
	imgLeftDisparity 		= cvCreateImage(sz,IPL_DEPTH_64F,1);
	imgRightDisparity 		= cvCreateImage(sz,IPL_DEPTH_64F,1);
	imgLeftOcclusions 		= cvCreateImage(sz,IPL_DEPTH_64F,1);
	imgRightOcclusions 	= cvCreateImage(sz,IPL_DEPTH_64F,1);

	//wrap all the input and output images in OpenCVImageAdapter, so that they can be
	//accessed by OpenVis3D
	
	leftResult.allocate(width, height);
	rightResult.allocate(width, height);
	
}

void depthCalculator::calculate(ofCvColorImage left, ofCvColorImage right, int minshift, int maxshift){

	CvSize sz;
	sz.height = left.getCvImage()->height;
	sz.width  = left.getCvImage()->width;
	
	
	OpenCVImageAdapter*ovaImg1 = new OpenCVImageAdapter(left.getCvImage());
	OpenCVImageAdapter*ovaImg2 = new OpenCVImageAdapter(right.getCvImage());
	OpenCVImageAdapter*ovaLeftDisp = new OpenCVImageAdapter(imgLeftDisparity);
	OpenCVImageAdapter*ovaRightDisp = new OpenCVImageAdapter(imgRightDisparity);
	OpenCVImageAdapter*ovaLeftOcc = new OpenCVImageAdapter(imgLeftOcclusions);
	OpenCVImageAdapter*ovaRightOcc = new OpenCVImageAdapter(imgRightOcclusions);

	
    //create Birchfield-Tomasi local matcher and set its default parameter alpha to 20.0
    BTLocalMatcherT<double> btmatcher;
    double alpha[] = {20.0};
    btmatcher.setParams(1,alpha);

    //create global diffusion-based stereo algorithm instance
    OvStereoDiffuseMatcherT<double> stereoDiffuseMatcher;

    //create general stereo algorithm execution manager instance
    OvStereoT<double> stereoManager;
    stereoManager.setLocalImageMatcher(btmatcher); //set local matcher to Birchfield-Tomasi
    stereoManager.setGlobalMatcher(stereoDiffuseMatcher); //set global stereo algorithm

    printf("\nRunning stereo ...\n");

    //EXECUTE stereo matching
    stereoManager.doStereoMatching(*ovaImg1, *ovaImg2, minshift, maxshift, *ovaLeftDisp, *ovaRightDisp, *ovaLeftOcc, *ovaRightOcc);
	
	printf("\ndone Running stereo ...\n");

    //DISPLAY RESULTS

    //BEGIN: rescale disparity maps to range (0,1) so that they can be displayed by OpenCV
    OvImageT<double> ovtleftDisp, ovtrightDisp;
    ovtleftDisp.copyFromAdapter(*ovaLeftDisp);
    ovtrightDisp.copyFromAdapter(*ovaRightDisp);

    ovtleftDisp = (ovtleftDisp-minshift)/(maxshift-minshift);
    ovtrightDisp = (ovtrightDisp+maxshift)/(-minshift+maxshift);

    ovtleftDisp.copyToAdapter(*ovaLeftDisp);
    ovtrightDisp.copyToAdapter(*ovaRightDisp);
    
    
    cvCvtScale(imgRightDisparity,leftResult.getCvImage(),255); 
    cvCvtScale(imgRightDisparity,rightResult.getCvImage(),255); 
    
    
    /*
    //try this, maybe faster?
    //http://www.itu.dk/stud/projects_f2004/handtracking/OpenCV/apps/StereoDemo/main.cpp
    cvFindStereoCorrespondence( leftImage, rightImage, CV_DISPARITY_BIRCHFIELD, depthImage, 50, 15, 3, 6, 8, 15 );
    */
   
    
    if(ovaImg1) delete ovaImg1;
    if(ovaImg2) delete ovaImg2;
    if(ovaLeftDisp) delete ovaLeftDisp;
    if(ovaRightDisp) delete ovaRightDisp;
    if(ovaLeftOcc) delete ovaLeftOcc;
    if(ovaRightOcc) delete ovaRightOcc;
	

}